<?php if (!defined('PmWiki')) exit();

/*

    jticker - show a simple list as a news ticker (uses JavaScript!)

    This file is jticker.php;  you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
    by the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.  See pmwiki.php for full details.
    
    To use this recipe, simply copy it into the cookbook/ directory, and
    add the following line to a local customization:

        include_once("$FarmD/cookbook/jticker.php");
        
    To include a list as a ticker, use:
    
       (:pagelist trail=<TrailPage> fmt=jticker:)

    This will scan the page <TrailPage> for lists and use the combined
    list to fill the ticker. Tweak the style.css to fit your needs.

    To use an RSS feed as a ticker, use:
    
       (:jtickerRSS feed=<Feed-URL> [cachetime=<time>] [cachename=<Name>] [maxitems=<Number>] :)
       
    Copyright 2011, 2014 Bianka Martinovic

*/

global $PubDirUrl;
global $_debug;
global $debug_info;

$_debug = false;
$_debug_info = array();

SDV($RecipeInfo['jticker']['Version'], '2014-10-29');
SDV($FPLFunctions['jticker'], 'jticker');

// URL to load the jQuery Core from
SDV(
    $jtickerapiurl,
    'https://ajax.googleapis.com/ajax/libs/jquery/1.5.1/jquery.min.js'
);
// Animation speed (higher values = slower)
SDV(
    $jtickerspeed,
    2000
);
// Time to show each item (higher values = longer)
SDV(
    $jtickertimeout,
    5000
);
// Easing animation type
SDV(
    $jtickereasing,
    "linear"
);
// Ticker height; use CSS or 'auto'
SDV(
    $jtickerheight,
    "1em"
);

Markup_e(
    'jtickerRSS',
    'fulltext',
    '/\\(:jtickerRSS (.*?):\\)/i',
    'jtickerRSS("$m[0]")'
);

$HTMLHeaderFmt[] = '<link rel="stylesheet" type="text/css" href="'.$PubDirUrl.'/jticker/style.css" />' . "\n"
                 . '<script charset="utf-8" src="'.$jtickerapiurl.'" type="text/javascript"></script>' . "\n"
                 . '<script charset="utf-8" src="'.$PubDirUrl.'/jticker/jquery.easing.js" type="text/javascript"></script>' . "\n"
                 . '<script charset="utf-8" src="'.$PubDirUrl.'/jticker/jquery.innerfade.js" type="text/javascript"></script>' . "\n"
                 . '<script charset="utf-8" type="text/javascript">' . "\n"
                 . '  if ( typeof jQuery != "undefined" ) {' . "\n"
                 . '    jQuery(document).ready(function($) {' . "\n"
                 . '      $("#scroll").innerfade({' . "\n"
                 . '        speed: '.$jtickerspeed.',' . "\n"
                 . '        easing: "'.$jtickereasing.'",' . "\n"
                 . '        timeout: "'.$jtickertimeout.'",' . "\n"
                 . '        containerheight: "'.$jtickerheight.'"' . "\n"
                 . '      });' . "\n"
                 . '    });' . "\n"
                 . '  }' . "\n"
                 . '</script>' . "\n"
                 ;

/*
    Main function
*/
function jticker($pagename, &$matches, $opt) {

    global $jtickerapiurl,
           $PubDirUrl,
           $_debug_info;

    list( $id, $t ) = __jtickerReadTrail( $pagename, $opt['trail'] );

    $_debug_info[] = sprintf('reading trail from page [%s], trail option [%s], id [%s]', $pagename, $opt['trail'], $id);
    $_debug_info[] = var_export($t,1);
    
    $output  = array();
    $depth   = $t[$id]['depth'];
    $isfirst = true;

    foreach( $t as $current ) {
        if ( $current['spacer'] ) {
            continue;
        }
        ## add current item to the list
        $output[] = ''
                  . str_repeat('*', $current['depth'])
                  . ( $isfirst ? ' %list id=scroll% ' : NULL )
                  . " {$current['markup']}";
        $isfirst  = false;
    }

    return implode( "\n", $output );

}   // end function jticker()

/*
    Fill the ticker using an RSS feed
    Based on "PMinlineRSS"
*/
function jtickerRSS($opts) {

    global $FarmD, $_debug_info, $_debug;

    $_debug_info[] = 'options:<br />'.var_export($opts,1);

    $cachepath     = $FarmD.'/pub/';                    // Path to directory for cache files
    $fileprefix    = 'cache_';                  // What cache file names get prefixed with
  
    $args          = ParseArgs($opts);
    $_debug_info[] = 'parsed args:<br />'.var_export($args,1);

    $p_feed        = $args['feed'];
    $p_cachetime   = 0;
    $p_cachetime   = $args['cachetime'];
    $p_cachename   = $args['cachename'];
    $p_maxitems    = $args['maxitems'];
    $cached        = FALSE;
    $xml           = "";

    if ($p_cachetime == 0) { $p_cachetime = 30; }

    // If a caching file is specified, see if we should just use it
    if (!$p_cachename == "") {
        $cachepathname = $cachepath . $fileprefix . $p_cachename . ".xml";
        if (file_exists($cachepathname)) {
            $age = time() - filectime($cachepathname);
            if ($age <= ($p_cachetime * 60)) {
                $xml = file_get_contents($cachepathname);
                $cached = TRUE;
            }
        }
    }

    if(stristr($p_feed,"http:") && substr_count($p_feed,'&amp;'))
        $p_feed = str_ireplace('&amp;','&',$p_feed);

    // We got nothing from cache (or it didn't exist or wasn't specified)
    if ($xml == "") {
        // Use CURL because, well, it's a tiny bit safer.
        if (function_exists('curl_init') and stristr($p_feed,"http:") ) {
            $curl_handle = curl_init();
            curl_setopt($curl_handle,CURLOPT_URL,$p_feed);
            curl_setopt($curl_handle,CURLOPT_CONNECTTIMEOUT,10);
            curl_setopt($curl_handle,CURLOPT_RETURNTRANSFER,1);
            $curlxml = curl_exec($curl_handle);
            curl_close($curl_handle);
            if (empty($curlxml))  {
                $_debug_info[] = "Error reading feed $p_feed using curl.";
                $writefile = FALSE;
            } else {
                $xml = $curlxml;
                $writefile = TRUE;
            }
        }
        else {
            //  If CURL is giving you problems, use the line below instead.
            $filegetxml = @file_get_contents($p_feed);
            if (empty($filegetxml)) {
                $_debug_info[] = "Error reading feed $p_feed using file_get.";
                $writefile = FALSE;
            }
            else {
                $xml = $filegetxml;
                $writefile = TRUE;
            }
        }
        if ($writefile && (!$p_cachename == "")) {
            if (!($handle = fopen($cachepathname,'w')))
                $_debug_info[] = "Error opening $cachepathname - possible permissions issue - directory permissions are " . substr(sprintf('%o', fileperms($cookbookpath)), -4);
            fwrite($handle,$xml);
            fclose($handle);
        }
    }

    // XML feed [$cachename] is empty.
    if (strlen($xml) < 1)
    {
        $_debug_info[] = "XML feed [$cachename] is empty.";
        if($_debug) echo implode("<br />\n",$_debug_info);
        return;
    }

    
    if (PHP_VERSION >= 5) {
        $xslt = new xsltProcessor;
        $XSL  = new DOMDocument();
        $XSL->load( $FarmD.'/pub/jticker/jticker.xslt' );
        $xslt->importStylesheet( $XSL );
        $xmld = new DomDocument();
        $xmld->loadXML($xml);
        $xslt_result = $xslt->transformToXML($xmld);
    }
    
    if ( $p_maxitems && is_numeric($p_maxitems) && $p_maxitems > 0 && $p_maxitems < 1000 ) {
        $temp   = preg_split( "#\r?\n#", $xslt_result );
        $result = implode( "\n", array_slice( $temp, 0, $p_maxitems+1 ) );
    }
    else {
        $result = $xslt_result;
    }

    if ( $p_maxitems && is_numeric($p_maxitems) && $p_maxitems > 0 && $p_maxitems < 1000 ) {
        $temp = preg_split( "#\r?\n#", $xslt_result );
        $result = implode( "\n", array_slice( $temp, 0, $p_maxitems+1 ) );
    }
    else {
        $result = $xslt_result;
    }
    
    // add id to first item
    $result = preg_replace( '#\*#', '* %list id=scroll% ', $result, 1 );

    return $result;

}

/*
    Filter out the list items
    This function was original created for DynTrailMenu
*/
function __jtickerReadTrail( $pagename, $trailname ) {

    global $SuffixPattern, $GroupPattern, $WikiWordPattern;

    $trailname = MakePageName($pagename,$trailname);
    $trailpage = ReadPage($trailname, READPAGE_CURRENT);

    if (!$trailpage) return false;

    $t         = array();
    $n         = 0;
    $listitems = array();

    $pattern    = "(\\[\\[([^|\\]]*)\\|?\\s*(.*?)\\s*\\]\\])($SuffixPattern) (.*)
                  | ($GroupPattern([\\/.])?$WikiWordPattern) (.*)";

    foreach(
        explode(
            "\n",
            htmlspecialchars(@$trailpage['text'], ENT_NOQUOTES)
        ) as $x)
    {

        ## leave blank lines as spacer
        if ( preg_match( "/^\s*$/", $x, $match ) ) {

            if ( $n == 0 ) { continue; }
            $listitems[$n]['id']     = $n;
            $listitems[$n]['spacer']++;

            $n++;
            continue;
        }

        ## is it a list item?
        if ( !preg_match( '/^([#*:]+)\s*(.*)/x', $x, $match ) ) {
            continue;
        }

        $item = ltrim(rtrim($match[2]));
        $listitems[$n]['id'] = $n;

        ## is it a wiki link?
        if (
            preg_match(
                "/$pattern/x",
                $item,
                $wikilink)
            )
        {
            $listitems[$n]['pagename'] = preg_replace( '#/#', '.', ltrim(rtrim($wikilink[2])) );
            $listitems[$n]['markup']   = preg_replace( "/\\\\(?>(\\\\*))$/e", "\n", ltrim(rtrim($wikilink[1])) );
            $listitems[$n]['wikilink']++;

            #if ( $listitems[$n]['pagename'] == $pagename ) {
            if ( ! strcasecmp( $listitems[$n]['pagename'], $pagename ) ) {
                $id = $n;
            }
        }

        ## is it a free link?
        elseif (
            preg_match(
                "/(\\[\\[\\s*(.*?)\\]\\])($SuffixPattern)(.*)/x",
                $item,
                $freelink )
            )
        {
            $listitems[$n]['markup']   = preg_replace( "/\\\\(?>(\\\\*))$/e", "\n", ltrim(rtrim($freelink[1])) );
            $listitems[$n]['freelink']++;
        }
        ## something else
        else {
            $text = ltrim(rtrim($match[2]));
            $listitems[$n]['markup']   = preg_replace( "/\\\\(?>(\\\\*))$/e", "\n", $text );
        }

        $depth = strlen( $match[1] );

        $listitems[$n]['depth']   = $depth;

        for( $i=$depth;$i<10;$i++) { $d[$i] = $n; }

        if ( $depth > 1 ) {

            $parent = @$d[$depth-1];

            $listitems[$n]['parent'] = $parent;
            $listitems[$n]['path']   = $listitems[$parent]['path'];
            $listitems[$n]['path'][$parent]++;

            $listitems[$parent]['children']++;

        }

        $n++;

    }

    return array( $id, $listitems );

}   // end function __jtickerReadTrail()

?>